import { eventWithTime, listenerHandler } from '@highlight-run/rrweb/dist/types';
import { FirstLoadListeners } from './listeners/first-load-listeners';
import { ConsoleMethods } from './listeners/console-listener';
import { Sdk, PushPayloadMutationVariables } from './graph/generated/operations';
import 'clientjs';
import { RequestResponsePair } from './listeners/network-listener/utils/models';
import { FeedbackWidgetOptions } from './ui/feedback-widget/feedback-widget';
export declare const HighlightWarning: (context: string, msg: any) => void;
declare class Logger {
    debug: boolean | undefined;
    constructor(debug?: boolean);
    log(...data: any[]): void;
}
export declare type DebugOptions = {
    clientInteractions?: boolean;
    domRecording?: boolean;
};
export declare type NetworkRecordingOptions = {
    /**
     * Enables recording of network requests.
     * The data includes the URLs, the size of the request, and how long the request took.
     * @default true
     */
    enabled?: boolean;
    /**
     * This enables recording XMLHttpRequest and Fetch headers and bodies.
     * @default false
     */
    recordHeadersAndBody?: boolean;
    /**
     * Request and response headers where the value is not recorded.
     * The header value is replaced with '[REDACTED]'.
     * These headers are case-insensitive.
     * `recordHeadersAndBody` needs to be enabled.
     * @example
     * networkHeadersToRedact: ['Secret-Header', 'Plain-Text-Password']
     */
    networkHeadersToRedact?: string[];
    /**
     * URLs to not record headers and bodies for.
     * To disable recording headers and bodies for all URLs, set `recordHeadersAndBody` to `false`.
     * @default ['https://www.googleapis.com/identitytoolkit', 'https://securetoken.googleapis.com']
     */
    urlBlocklist?: string[];
};
export declare type SessionShortcutOptions = false | string;
export declare type IntegrationOptions = {
    mixpanel?: MixpanelIntegrationOptions;
    amplitude?: AmplitudeIntegrationOptions;
};
export interface MixpanelIntegrationOptions {
    projectToken: string;
}
export interface AmplitudeIntegrationOptions {
    apiKey: string;
}
export declare type HighlightClassOptions = {
    organizationID: number | string;
    debug?: boolean | DebugOptions;
    backendUrl?: string;
    tracingOrigins?: boolean | (string | RegExp)[];
    disableNetworkRecording?: boolean;
    networkRecording?: boolean | NetworkRecordingOptions;
    disableConsoleRecording?: boolean;
    consoleMethodsToRecord?: ConsoleMethods[];
    enableSegmentIntegration?: boolean;
    enableStrictPrivacy?: boolean;
    enableCanvasRecording?: boolean;
    firstloadVersion?: string;
    environment?: 'development' | 'production' | 'staging' | string;
    appVersion?: string;
    sessionShortcut?: SessionShortcutOptions;
    feedbackWidget?: FeedbackWidgetOptions;
    sessionSecureID?: string;
};
/**
 * Subset of HighlightClassOptions that is stored with the session. These fields are stored for debugging purposes.
 */
declare type HighlightClassOptionsInternal = Omit<HighlightClassOptions, 'firstloadVersion'>;
declare type PropertyType = {
    type?: 'track' | 'session';
    source?: Source;
};
declare type Source = 'segment' | undefined;
export declare type SessionData = {
    sessionID: number;
    sessionSecureID: string;
    projectID: number;
    sessionStartTime?: number;
    lastPushTime?: number;
    userIdentifier?: string;
    userObject?: Object;
};
export declare class Highlight {
    options: HighlightClassOptions;
    /** Determines if the client is running on a Highlight property (e.g. frontend). */
    isRunningOnHighlight: boolean;
    /** Verbose project ID that is exposed to users. Legacy users may still be using ints. */
    organizationID: string;
    graphqlSDK: Sdk;
    events: eventWithTime[];
    xhrNetworkContents: RequestResponsePair[];
    fetchNetworkContents: RequestResponsePair[];
    tracingOrigins: boolean | (string | RegExp)[];
    networkHeadersToRedact: string[];
    urlBlocklist: string[];
    sessionData: SessionData;
    ready: boolean;
    state: 'NotRecording' | 'Recording';
    /**
     * The number of requests to public graph that have failed in a row.
     */
    numberOfFailedRequests: number;
    logger: Logger;
    disableNetworkRecording: boolean;
    enableRecordingNetworkContents: boolean;
    enableSegmentIntegration: boolean;
    enableStrictPrivacy: boolean;
    enableCanvasRecording: boolean;
    debugOptions: DebugOptions;
    listeners: listenerHandler[];
    firstloadVersion: string;
    environment: string;
    sessionShortcut: SessionShortcutOptions;
    /** The end-user's app version. This isn't Highlight's version. */
    appVersion: string | undefined;
    _optionsInternal: HighlightClassOptionsInternal;
    _backendUrl: string;
    _recordingStartTime: number;
    _isOnLocalHost: boolean;
    _onToggleFeedbackFormVisibility: () => void;
    _firstLoadListeners: FirstLoadListeners;
    pushPayloadTimerId: ReturnType<typeof setTimeout> | undefined;
    feedbackWidgetOptions: FeedbackWidgetOptions;
    hasSessionUnloaded: boolean;
    hasPushedData: boolean;
    static create(options: HighlightClassOptions): Highlight;
    constructor(options: HighlightClassOptions, firstLoadListeners?: FirstLoadListeners);
    _reset(): Promise<void>;
    _initMembers(options: HighlightClassOptions): void;
    identify(user_identifier: string, user_object?: {}, source?: Source): Promise<void>;
    _stringifyProperties(properties_object: any, type: 'session' | 'track' | 'user'): any;
    pushCustomError(message: string, payload?: string): Promise<void>;
    consumeCustomError(error: Error, message?: string, payload?: string): Promise<void>;
    addProperties(properties_obj?: {}, typeArg?: PropertyType): Promise<void>;
    initialize(): Promise<void>;
    /**
     * Stops Highlight from recording.
     * @param manual The end user requested to stop recording.
     */
    stopRecording(manual?: boolean): void;
    getCurrentSessionTimestamp(): number;
    /**
     * Returns the current timestamp for the current session.
     */
    getCurrentSessionURLWithTimestamp(): string;
    getCurrentSessionURL(): string | null;
    toggleFeedbackWidgetVisibility(): void;
    addSessionFeedback({ timestamp, verbatim, user_email, user_name, }: {
        verbatim: string;
        timestamp: string;
        user_name?: string;
        user_email?: string;
    }): void;
    _save(): Promise<void>;
    _shouldSendRequest(): boolean;
    /**
     * This proxy should be used instead of rrweb's native addCustomEvent.
     * The proxy makes sure recording has started before emitting a custom event.
     */
    addCustomEvent<T>(tag: string, payload: T): void;
    _sendPayload({ isBeacon, sendFn, }: {
        isBeacon: boolean;
        sendFn: (payload: PushPayloadMutationVariables) => Promise<any>;
    }): Promise<void>;
}
declare global {
    interface Console {
        defaultLog: any;
        defaultError: any;
        defaultWarn: any;
        defaultDebug: any;
    }
}
export {};
